﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Survive
{
    class Bullet : Sprite
    {
        // bullet moving
        const int speed = 40;
        Vector2 direction;

        // bullet drawing/scaling
        Vector2 originalPosition;
        Rectangle destinationRect;
        Rectangle sourceRect;
        bool toDraw = true;

        // distance variables
        Vector2 distance;
        float distanceLength;

        // color variables
        byte red = 255;
        byte green = 255;
        byte greenRate = 45;
        byte blue = 255;
        byte blueRate = 35;
        byte alpha = 190;

        // time variables

        public bool ToDraw
        {
            get { return toDraw; }
            set { toDraw = value; }
        }

        public Bullet(Texture2D texture, Vector2 position, Point frameSize, Point currentFrame, Point sheetSize, Color color, float scale, float rotation, Vector2 origin, float layerDepth, SpriteEffects spriteEffects, Vector2 collisionOffset, int msPerFrame, Vector2 direction)
            : base(texture, position, frameSize, currentFrame, sheetSize, color, scale, rotation, origin, layerDepth, spriteEffects, collisionOffset, msPerFrame)
        {
            originalPosition = position;
            sourceRect = new Rectangle(0, 0, frameSize.X, frameSize.Y);
            destinationRect = new Rectangle((int)position.X, (int)position.Y,(int)sourceRect.Width,(int)sourceRect.Height);

            this.direction = direction;
        }

        public override void Update(GameTime gameTime, Rectangle clientBounds)
        {
            // update position
            position += speed * direction;
 
            destinationRect.X = (int)position.X;
            destinationRect.Y = (int)position.Y;

            // check if the bullet has exited the screen
            if ((position.X + destinationRect.Height) < 0 || (position.X - destinationRect.Height) > clientBounds.Width)
            {
                toDraw = false;
                return;
            }

            if ((position.Y + destinationRect.Height) < 0 || (position.Y - destinationRect.Height) > clientBounds.Height)
            {
                toDraw = false;
                return;
            }

            // scale based on distance
            distance.X = MathHelper.Distance(originalPosition.X, position.X);
            distance.Y = MathHelper.Distance(originalPosition.Y, position.Y);
            distanceLength = (int)Math.Sqrt((distance.X*distance.X)+(distance.Y)*(distance.Y));
            if (destinationRect.Height < 100)
            {
                destinationRect.Height *= 2; 
            }
            destinationRect.X += destinationRect.Height * (int)direction.X;
            destinationRect.Y += destinationRect.Height * (int)direction.Y;
             //implement scaling here

            // change color based on time elapsed
            if (blue > 0)
            {
                if (blue - blueRate < 0)
                    blue = 0;
                else blue -= blueRate;
            }
            else
            {
                if (green - greenRate < 0)
                    green = 0;
                else green -= greenRate;
            }
                

            color = new Color(red, green, blue, alpha);
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(texture, destinationRect, sourceRect, color, rotation, Vector2.Zero /*new Vector2(destinationRect.Width / 2, destinationRect.Height / 2)*/, spriteEffects, layerDepth);
            //spriteBatch.DrawString(font, "Position X: " + distance.X + "\nPosition Y: " + distance.Y, new Vector2(destinationRect.X + 20, destinationRect.Y + 20), Color.YellowGreen);
        }
    }
}
