using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Survive
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;

        List<Texture2D> backgrounds = new List<Texture2D>();

        List<Texture2D> tiles = new List<Texture2D>();
        static int tileWidth = 100;
        static int tileHeight = 100;
        Rectangle tileRect = new Rectangle(0, 0, tileWidth, tileHeight);

        Camera camera;

        int[,] map = {
                         {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
                         {0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0},
                         {0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0},
                         {0,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},
                         {0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0},
                         {0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0},
                         {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
                     };



        enum GameStates
        {
            Intro,
            Setup,
            Play,
            Pause,
            Lose
        };

        GamePadState oldGPState;
        GamePadState newGPState;

        GameStates gameState;
        int milliSecondsElapsed;

        Player player;
        SpriteFont gogoZombie;
        Texture2D playerSprite;
        Texture2D bulletSprite;

        Zombie testZombie;
        Texture2D zombieSprite;

        Texture2D logo;
        Vector2 logoPosition;
        Vector2 logoCenter;
        byte logoAlpha = 0;
        int logoMilliseconds = 3500;
        bool logoCentered = false;
        bool logoShown = false;

        Texture2D survive;
        Rectangle surviveRect;

        Texture2D gameOverTexture;
        Rectangle gameOverRect;

        Texture2D light;

        Texture2D fadeOverlay;
        byte fadeOverlayAlpha = 255;

        private Point VectorToCell(Vector2 vector)
        {
            Point V2C = new Point();
            
            if (vector.X <= 0)
                V2C.X = 0;
            else
                V2C.X = (int)(vector.X / tileWidth);

            if (vector.Y <= 0)
                V2C.Y = 0;
            else
                V2C.Y = (int)(vector.Y / tileHeight);

            return V2C;
        }

        private Vector2 ViewPortVector()
        {
            return new Vector2(Window.ClientBounds.Width + (tileWidth),
                Window.ClientBounds.Height + (tileHeight));
        }

        public void DrawMap(SpriteBatch spritebatch)
        {
            Point cameraPoint = VectorToCell(camera.Position);
            Point viewPoint = VectorToCell(camera.Position + ViewPortVector());
            Point min = cameraPoint;
            Point max = new Point();

            max.X = (int)Math.Min(viewPoint.X, map.GetLength(1));
            max.Y = (int)Math.Min(viewPoint.Y, map.GetLength(0));


            for (int y = min.Y; y < max.Y; y++)
            {
                for (int x = min.X; x < max.X; x++)
                {
                    tileRect.X = x * tileWidth - (int)camera.Position.X;
                    tileRect.Y = y * tileHeight - (int)camera.Position.Y;

                    spriteBatch.Draw(
                        tiles[map[y, x]],
                        tileRect,
                        Color.White);
                }
            }
        }

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);

            graphics.PreferredBackBufferHeight = 720;
            graphics.PreferredBackBufferWidth = 1280;

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            gameState = GameStates.Intro;

            oldGPState = GamePad.GetState(PlayerIndex.One);
            camera = new Camera(new Vector2(300, 300), 8);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // intro assets
            logo = Content.Load<Texture2D>(@"Intro/GhitaGames");
            logoCenter = new Vector2((Window.ClientBounds.Width - logo.Width) / 2, (Window.ClientBounds.Height-logo.Height) / 2);
            logoPosition = new Vector2(-logo.Width, logoCenter.Y);


            // tiles
            tiles.Add(Content.Load<Texture2D>(@"Tiles/null tile"));
            tiles.Add(Content.Load<Texture2D>(@"Tiles/grass tile 1"));

            // backgrounds
            backgrounds.Add(Content.Load<Texture2D>(@"Backgrounds/background 1"));
            backgrounds.Add(Content.Load<Texture2D>(@"Backgrounds/background 2"));
            fadeOverlay = Content.Load<Texture2D>(@"Backgrounds/fadeOverlay");
            light = Content.Load<Texture2D>(@"Backgrounds/light");
                
           // player
            playerSprite = Content.Load<Texture2D>(@"Sprites/shooter");
            bulletSprite = Content.Load<Texture2D>(@"Sprites/bullet");
            Vector2 origin = new Vector2(50, 78);
            player = new Player(playerSprite, new Vector2((int)((Window.ClientBounds.Width + origin.X)/2), (int)((Window.ClientBounds.Height + origin.Y)/2)), new Point(100, 100), new Point(0, 0), new Point(0, 0), Color.White, 1.0f, 0.0f, origin, 1.0f, SpriteEffects.None, new Vector2(16, 6), 100 /*, new SoundBank()*/);
            player.BulletTexture = bulletSprite;

            //zombie
            Random posOffset = new Random();
            int offsetx = posOffset.Next(-100, 100);
            int offsety = posOffset.Next(-100, 100);
            zombieSprite = Content.Load<Texture2D>(@"Sprites/zombie");
            testZombie = new Zombie(zombieSprite, new Vector2((int)((Window.ClientBounds.Width + origin.X) / 2)+offsetx, (int)((Window.ClientBounds.Height + origin.Y) / 2)+offsety), new Point(100, 100), new Point(0, 0), new Point(0, 0), Color.White, 1.0f, 0.0f, origin, 1.0f, SpriteEffects.None, new Vector2(16, 6), 100 /*, new SoundBank()*/);
            

            // title
            survive = Content.Load<Texture2D>(@"Sprites/Survive");
            surviveRect = new Rectangle((int)(Window.ClientBounds.Width + origin.X) / 2, (int)((Window.ClientBounds.Height + origin.Y) / 2) + 50, survive.Width, survive.Height);

            // game over
            gameOverTexture = Content.Load<Texture2D>(@"Sprites/game-over");
            gameOverRect = new Rectangle((int)Window.ClientBounds.Center.X - gameOverTexture.Width, (int)Window.ClientBounds.Center.Y - gameOverTexture.Height, gameOverTexture.Width, gameOverTexture.Height);

            // font
            gogoZombie = Content.Load<SpriteFont>(@"Font/gogozombie");
            player.Font = gogoZombie;
        }

        /// <summary>
        /// UnloadContent can suck it
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            newGPState = GamePad.GetState(PlayerIndex.One);

            // Allows the game to exit
            if (newGPState.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            milliSecondsElapsed = gameTime.ElapsedGameTime.Milliseconds;

            switch (gameState)
            {
                case GameStates.Intro:
                    {
                        if (!(newGPState.Buttons.Start == ButtonState.Pressed && oldGPState.Buttons.Start != ButtonState.Pressed))
                        {
                            if (logoShown == true)
                            {
                                logoAlpha -= 5;
                                if (logoAlpha == 0)
                                    gameState = GameStates.Setup;
                            }
                            else
                            {
                                if (logoCentered == true)
                                {
                                    //start timer
                                    logoMilliseconds -= milliSecondsElapsed;
                                    if (logoMilliseconds <= 0)
                                        logoShown = true;
                                }
                                else
                                {
                                    if (logoPosition.X < logoCenter.X)
                                    {
                                        logoPosition.X += 20;
                                        logoAlpha++;
                                    }
                                    else if (logoPosition.X >= logoCenter.X)
                                    {
                                        logoPosition.X = logoCenter.X;
                                        logoAlpha = 255;
                                        logoCentered = true;
                                    }
                                }
                            }
                        }
                        else
                            gameState = GameStates.Setup;

                       
                        
                        break;
                    }
                case GameStates.Setup:
                    {

                        // quick fade in
                        if (fadeOverlayAlpha > 0)
                        {
                            fadeOverlayAlpha -= 5;
                            if (fadeOverlayAlpha < 0)
                                fadeOverlayAlpha = 0;
                        }

                        camera.Update(newGPState);

                        player.Update(gameTime, Window.ClientBounds, newGPState, oldGPState);
                        player.Health = 4;

                        testZombie.Direction = new Vector2(testZombie.Position.X - camera.Position.X,testZombie.Position.Y - camera.Position.Y);
                        testZombie.Update(gameTime, Window.ClientBounds);

                        if (newGPState.IsButtonDown(Buttons.Start) && !oldGPState.IsButtonDown(Buttons.Start))
                            gameState = GameStates.Pause;
                       
                        break;
                    }
                case GameStates.Play:
                    {
                        break;
                    }
                case GameStates.Pause:
                    {
                        if (newGPState.IsButtonDown(Buttons.Start) && !oldGPState.IsButtonDown(Buttons.Start))
                            gameState = GameStates.Setup;

                        if (newGPState.IsButtonDown(Buttons.Y))
                            gameState = GameStates.Lose;

                        break;
                    }
                case GameStates.Lose:
                    {
                        if (newGPState.IsButtonDown(Buttons.Start) && !oldGPState.IsButtonDown(Buttons.Start))
                            gameState = GameStates.Intro;

                        break;
                    }
            };

            oldGPState = newGPState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);

            switch (gameState)
            {
                case GameStates.Intro:
                    {
                        spriteBatch.Draw(
                            logo, 
                            new Rectangle((int)logoPosition.X, (int)logoPosition.Y, logo.Width, logo.Height), 
                            new Color(Color.White, logoAlpha)
                            );
                        break;
                    }
                case GameStates.Setup:
                    {
                        // draw stars with parallax effect eventually 
                        foreach (Texture2D t in backgrounds)
                        {
                            spriteBatch.Draw(t, new Rectangle(0, 0, t.Width, t.Height), Color.White);
                        }

                        // draw tiles
                        DrawMap(spriteBatch);

                        // draw player & bullets
                        player.Draw(spriteBatch);
                        testZombie.Draw(spriteBatch, camera.Position);
                        
                        // drive "SURVIVE" title
                        spriteBatch.Draw(survive, new Rectangle(surviveRect.X - (int)camera.Position.X, surviveRect.Y - (int)camera.Position.Y, surviveRect.Width, surviveRect.Height), Color.White);

                        spriteBatch.Draw(light, new Rectangle(0, 0, 1280, 720), Color.White);

                        // fade in overlay
                        if (fadeOverlayAlpha != 0)
                            spriteBatch.Draw(fadeOverlay, new Rectangle(0, 0, 1280, 720), new Color(Color.White, fadeOverlayAlpha));

                       break;
                    }
                case GameStates.Play:
                    {
                        break;
                    }
                case GameStates.Pause:
                    {
                        DrawMap(spriteBatch);
                        player.Draw(spriteBatch);
                        spriteBatch.Draw(fadeOverlay, new Rectangle(0, 0, 1280, 720), new Color(Color.White, 155));
                        break;
                    }
                case GameStates.Lose:
                    {
                        spriteBatch.Draw(fadeOverlay, new Rectangle(0, 0, 1280, 720), new Color(Color.DarkSalmon, 255));
                        spriteBatch.Draw(gameOverTexture, gameOverRect, Color.White);
                        break;
                    }
            };

            spriteBatch.End();

            base.Draw(gameTime);
        }
       
    

    }
}
