﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;
using Survive;

namespace Survive
{
    class Player : Sprite
    {
        // vitals
        int health;

        // aiming
        float rotationSpeed;
        float rotationTarget;
        float elapsed;
        float circle;
        float difference;

        Vector2 direction = new Vector2(0, -1);

        // bullets
        List<Bullet> bulletList = new List<Bullet>();
        Texture2D bulletTexture;

        public int Health
        {
            get { return health; }
            set { health = value; }
        }

        public Texture2D BulletTexture
        {
            get { return bulletTexture; }
            set { bulletTexture = value; }
        }

        public Player(Texture2D texture, Vector2 position, Point frameSize, Point currentFrame, Point sheetSize, Color color, float scale, float rotation, Vector2 origin, float layerDepth, SpriteEffects spriteEffects, Vector2 collisionOffset, int msPerFrame/*, SoundBank soundBank*/)
            : base(texture, position, frameSize, currentFrame, sheetSize, color, scale, rotation, origin, layerDepth, spriteEffects, collisionOffset, msPerFrame /*, soundBank*/)
        {
            rotationSpeed = (float)MathHelper.TwoPi;
            circle = MathHelper.TwoPi;
        }

        public void Update(GameTime gameTime, Rectangle clientBounds, GamePadState gpState, GamePadState prevGPState)
        {
            elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;

            // handle right thumbstick for rotation/aiming
            gpState.ThumbSticks.Right.Normalize();

            if (gpState.ThumbSticks.Right.Length() != 0)
            {
                rotationTarget = (float)Math.Atan2(gpState.ThumbSticks.Right.Y, -gpState.ThumbSticks.Right.X) -MathHelper.PiOver2;
                difference = MathHelper.WrapAngle(rotation - rotationTarget);

                // clockwise rotation
                if (difference < -0.1f)
                    rotation += (elapsed * rotationSpeed) % circle;

                // counter-clockwise rotation
                if (difference > 0.1f)
                    rotation -= (elapsed * rotationSpeed) % circle;

                rotation = MathHelper.WrapAngle(rotation);

                direction.X = (float)Math.Round(Math.Sin(rotation), 2); 
                direction.Y = -(float)Math.Round(Math.Cos(rotation), 2);
            }

            // handle shooting
            if (gpState.IsButtonDown(Buttons.RightTrigger) && !prevGPState.IsButtonDown(Buttons.RightTrigger))
            {
                bulletList.Add(new Bullet(bulletTexture, new Vector2(60*direction.X + position.X, 60*direction.Y + position.Y) , new Point(8, 8), Point.Zero, Point.Zero, Color.White, 1.0f, rotation, Vector2.Zero, 0.75f, spriteEffects, new Vector2(2.5f, 0.0f), 100, direction));
                bulletList.Last<Bullet>().Font = font;
            }

            // update bullets

                for (int i = 0; i < bulletList.Count; i++)
                {
                    bulletList[i].Update(gameTime, clientBounds);
                    
                    if (bulletList[i].ToDraw == false)
                        bulletList.RemoveAt(i);
                }

            // check collisions



            base.Update(gameTime, clientBounds);
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            // debug text
            //spriteBatch.DrawString(font, "Rotation target: " + rotationTarget, new Vector2(900, 20), Color.DeepPink);
            //spriteBatch.DrawString(font, "Rotation actual: " + rotation, new Vector2(900, 65), Color.Purple);
            //spriteBatch.DrawString(font, "Rt thumb X: " + direction.X, new Vector2(900, 110), Color.Crimson);
            //spriteBatch.DrawString(font, "Rt thumb Y: " + direction.Y, new Vector2(900, 155), Color.DarkSalmon);

            // draw the bullets
                foreach (Bullet b in bulletList)
                {
                    b.Draw(spriteBatch);
                }
            // draw the player 
            base.Draw(spriteBatch);
        }
    }
}
