﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;

namespace Survive
{
    abstract class Sprite
    {
        protected Texture2D texture;
        protected SpriteFont font;

        protected Vector2 position;
        //protected Vector2 speed;
        protected Vector2 origin;

        protected Point frameSize;
        protected Point currentFrame;
        protected Point sheetSize;
        protected Vector2 offset;

        protected Color color;

        protected float scale;
        protected float rotation;

        protected float layerDepth;

        protected SpriteEffects spriteEffects;

        protected int millisecondsPerFrame;
        protected int timeSinceLastFrame = 0;

        protected SoundBank soundBank;

        public SpriteFont Font
        {
            set { font = value; }
            get { return font; }
        }

        public float Rotation
        {
            set { rotation = value; }
            get { return rotation; }
        }

        public Sprite(Texture2D texture, Vector2 position, /*Vector2 speed,*/ Point frameSize, Point currentFrame, Point sheetSize, Color color, float scale, float rotation, Vector2 origin, float layerDepth, SpriteEffects spriteEffects,Vector2 offset, int millisecondsPerFrame /*, SoundBank soundBank*/)
        {
            this.texture = texture;
            this.position = position;
            //this.speed = speed;
            this.frameSize = frameSize;
            this.currentFrame = currentFrame;
            this.sheetSize = sheetSize;
            this.frameSize = frameSize;
            this.color = color;
            this.scale = scale;
            this.rotation = rotation;
            this.offset = offset;
           // this.soundBank = soundBank;
            this.origin = origin;
            
            this.layerDepth = layerDepth;
            this.spriteEffects = spriteEffects;
            this.millisecondsPerFrame = millisecondsPerFrame;

        }

        public virtual void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(
                texture,
                new Vector2((int)Math.Round(position.X,1), (int)Math.Round(position.Y,1)),
                new Rectangle(currentFrame.X * frameSize.X, currentFrame.Y * frameSize.Y, frameSize.X, frameSize.Y),
                color,
                rotation,
                origin,
                scale,
                spriteEffects,
                layerDepth
                );
        }

        public virtual void Update(GameTime gameTime, Rectangle clientBounds)
        {
            timeSinceLastFrame += gameTime.ElapsedGameTime.Milliseconds;
            if (timeSinceLastFrame > millisecondsPerFrame)
            {
                timeSinceLastFrame -= millisecondsPerFrame;
                ++currentFrame.X;

                if (currentFrame.X > sheetSize.X)
                {
                    currentFrame.X = 0;
                    ++currentFrame.Y;

                    if (currentFrame.Y > sheetSize.Y)
                        currentFrame.Y = 0;
                }
            }
        }

    }
}
